package main.oauth;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.NotImplementedException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import com.justep.util.ContextUtil;
import com.justep.util.SpringWebUtil;
import com.justep.util.net.AuthUtil;
import com.justep.util.net.ServiceUtil;

import main.bean.OauthUserInfo;
import main.config.OauthConfig;
import main.service.UaaService;

public abstract class OauthProcessor<U extends OauthUserInfo,C extends OauthConfig>{
	
	@Autowired
	UaaService uaaService;
	
	@Autowired
	C oauthConfig;
	
	
	//主执行链路
	public void process(String code,String location) {
		if(StringUtils.isBlank(code)) {
			throw new IllegalArgumentException("oauth回调code不能为空!");
		}
		
		if(StringUtils.isBlank(location)) {
			throw new IllegalArgumentException("oauth登录的location不能为空!");
		}
		String accessToken = this.onRetriveAccessToken(code);
		U oauthUserInfo = this.onRetriveOauthUserInfo(accessToken);
		String session = null;
		if(this.checkNeedLogin(oauthUserInfo)) {
			String[] shadowUserLoginInfo = this.onRegisteUser(oauthUserInfo, oauthConfig);
			session = this.onLogin(shadowUserLoginInfo);
		}
		this.onProcessReponse(session, location);
		this.onOauthSuccess(oauthUserInfo,session);
	}
	
	
	public abstract String onRetriveAccessToken(String code) ;
	
	public abstract U onRetriveOauthUserInfo(String accessToken);
	
	public boolean checkNeedLogin(U userInfo) {
		HttpServletRequest request = SpringWebUtil.getRequest();
		
		String	username = request.getHeader("X-Credential-Username");		
		if(StringUtils.isNotEmpty(username) && StringUtils.equals(userInfo.getUserName(), username)) {
			return false;
		}
		
		return true;
	}
	
	
	public String[] onRegisteUser(U userInfo,C oauthConfig) {
		return uaaService.registerShadowUser(
				oauthConfig.getClientId(), 
				oauthConfig.getClientId(), 
				userInfo.getOpenId(), 
				userInfo.getUserName(),
				userInfo.getNickname(),
				"oauth");		
		
	}
	
	
	public String onLogin(String[] shadowUserLoginInfo) {
		return AuthUtil.login(ServiceUtil.getServiceUrl("entry"), shadowUserLoginInfo[0], shadowUserLoginInfo[1]);
	}
	
	public void onProcessReponse(String session,String location) {
		HttpServletResponse response = SpringWebUtil.getResponse();
		
		if(StringUtils.isNotEmpty(session)) {
			Cookie cookie = new Cookie("user_session",session);
			cookie.setHttpOnly(false);
			if("https".equals(ContextUtil.getProtocol())) {
				cookie.setSecure(true);
			}
			cookie.setPath("/");
			cookie.setMaxAge(60*60*8);
			response.addCookie(cookie);
		}
		response.addHeader("location", location);
		response.setStatus(302);
	}
	
	//oauth登录成功后回调
	public void onOauthSuccess(U oauthUserInfo,String session) {};

	
}
