package main.oauth;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.message.BasicNameValuePair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.justep.util.ContextUtil;
import com.justep.util.SpringWebUtil;
import com.justep.util.net.RequestUtil;
import com.justep.util.net.ServiceUtil;

import main.bean.OauthUserInfo;
import main.config.OauthConfig;


@Service
public class OauthProcessorImpl extends OauthProcessor<OauthUserInfo,OauthConfig>{
	
	Logger logger = LoggerFactory.getLogger(OauthProcessorImpl.class);

	@Override
	public String onRetriveAccessToken(String code)  {
		String url = this.oauthConfig.getAccessTokenApiUrl();		
		
		List<BasicNameValuePair> body = new ArrayList<BasicNameValuePair>();
		
		body.add(new BasicNameValuePair("grant_type", "authorization_code"));
		body.add(new BasicNameValuePair("client_id", this.oauthConfig.getClientId()));
		body.add(new BasicNameValuePair("client_secret", this.oauthConfig.getClientSecret()));
		// body.add(new BasicNameValuePair("redirect_uri", this.oauthConfig.getRedirectUri()));
		body.add(new BasicNameValuePair("code", code));
		
		
		Map<String, String> headers = new HashMap<String, String>();
		headers.put("Content-Type", "application/x-www-form-urlencoded"); // 添加请求头
		
		UrlEncodedFormEntity urlEncodedFormEntity;
		try {
			urlEncodedFormEntity = new UrlEncodedFormEntity(body, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}	
		
		JSONObject result = ServiceUtil.post(url, urlEncodedFormEntity, headers, JSONObject.class);	
		logger.info("请求url:"+url);
		logger.info(JSON.toJSONString(result));
		if (result != null && result.containsKey("access_token")) {
			return result.getString("access_token");
		}
		throw new RuntimeException(JSON.toJSONString(result));
	}


	@Override
	public OauthUserInfo onRetriveOauthUserInfo(String accessToken) {		
		String url = this.oauthConfig.getUserInfoApiUrl();			
		
		List<BasicNameValuePair> body = new ArrayList<BasicNameValuePair>();
		
		body.add(new BasicNameValuePair("access_token", accessToken));
		
		
		UrlEncodedFormEntity urlEncodedFormEntity;
		try {
			urlEncodedFormEntity = new UrlEncodedFormEntity(body, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}

		
		Map<String, String> headers = new HashMap<String, String>();
		headers.put("Content-Type", "application/x-www-form-urlencoded"); // 添加请求头
		headers.put("X-Credential-User-Id", "system");
		
		JSONObject result = ServiceUtil.post(url, urlEncodedFormEntity, headers,JSONObject.class);
		
		logger.info(JSON.toJSONString(result));
		if(result.getIntValue("ret") == 0) {
			 String uid = result.getString("username");
			 OauthUserInfo userinfo = new OauthUserInfo();
			 userinfo.setOpenId(uid);
			 userinfo.setUserName(uid);
			 userinfo.setNickname(uid);
			 return userinfo;
		}
		throw new RuntimeException(JSON.toJSONString(result));
	}	
	
	
	@Override
	public void onOauthSuccess(OauthUserInfo oauthUserInfo, String session) {		
		super.onOauthSuccess(oauthUserInfo, session);
	}
	
	
	public boolean checkPermission() {
		return true;
	}
	
	public String getSsoCookie(String sessionId) {
		sessionId = "iamToken_" + sessionId;
		String url = this.oauthConfig.getUserInfoApiUrl();
		
		Map<String, String> params = new HashMap<String, String>();
		params.put("sessionId",sessionId);
		
		Map<String, String> headers = new HashMap<String, String>();
		
		JSONObject result = ServiceUtil.get(url, params,headers);
		
		return result.getString(sessionId);
	}
	
}
